package cz.zweistein.gw.predictor;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;

public class Guilds {
	private static Logger logger = Logger.getLogger(Guilds.class);
	
	private Map<String, Guild> guilds;
	
	private Guilds() {
		this.guilds = new HashMap<String, Guild>();
	}
	
	public Guild getGuildByName(String guildName) {
		Guild guild = this.guilds.get(guildName);
		
		if (guild == null) {
			guild = new Guild();
			guild.setName(guildName);
			this.guilds.put(guildName, guild);
		}
		
		return guild;
		
	}
	
	public boolean isKnown(String guildName) {
		return this.guilds.containsKey(guildName);
	}
	
	public List<Guild> getListOfPredictedOrder() {
		List<Guild> prediction = new ArrayList<Guild>(this.guilds.values());
		Collections.sort(prediction);
		return prediction;
	}
	
	public static Guilds getGuilds () throws ClientProtocolException, IOException {
		
		Guilds guilds = new Guilds();
		
		Calendar calNow = new GregorianCalendar();
		Calendar calTournament = new GregorianCalendar(2008, 0, 1);
		
		while (calTournament.getTimeInMillis() < calNow.getTimeInMillis()) { 
			
			String url = "http://www.guildwars.com/competitive/tournament" 
				+ calTournament.get(Calendar.YEAR) + "/"
				+ calTournament.getDisplayName(Calendar.MONTH, Calendar.LONG, Locale.US).toLowerCase()
				+ "/gvgresultselim.php";
			
			logger.info("Attempting to load "+url);
		
	        HttpClient httpclient = new DefaultHttpClient();
	        HttpGet httpget = new HttpGet(url); 
	        HttpResponse response = httpclient.execute(httpget);
	        
	        HttpEntity entity = response.getEntity();
	        String pageContent = EntityUtils.toString(entity);
	        if (pageContent.indexOf("We're sorry, the page you requested could not be found.") > 0) {
	        	logger.info("Page load failed.");
	        	break;
	        }
	        String resultTable = pageContent.substring(pageContent.indexOf("Elimination Results"), pageContent.indexOf("Swiss Rounds Results"));
	        
	        String regEx= "<td>(.+)</td><td>(.+)</td>";
	        
	        Pattern pattern = Pattern.compile(regEx);
	        Matcher matcher = pattern.matcher(resultTable);
	        
	        
	        while (matcher.find()) {
	        	int order = Integer.parseInt(matcher.group(1));
	        	String guildName = matcher.group(2).trim();
	        	
	        	Guild guild = guilds.getGuildByName(guildName);
	        	guild.addOrder(order);
	        }

	        calTournament.add(Calendar.MONTH, 1);
        
		}
		
		return guilds;
	}
	
	public static List<Guild> getPredicableGuilds(String userId, String userName, String password) throws ClientProtocolException, IOException {
		HttpClient httpclient = new DefaultHttpClient();
        
        HttpPost httpost = new HttpPost("https://www.guildwars.com/competitive/xunlai_tournament_house/login.php?lang=en");

        List <NameValuePair> nvps = new ArrayList <NameValuePair>();
        nvps.add(new BasicNameValuePair("continue", "View GvG Ballot"));
        nvps.add(new BasicNameValuePair("action", "insert"));
        nvps.add(new BasicNameValuePair("user_id", userId));
        nvps.add(new BasicNameValuePair("email_address", userName));
        nvps.add(new BasicNameValuePair("password", password));

        httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));

        HttpResponse response = httpclient.execute(httpost);
        HttpEntity entity = response.getEntity();
        String pageContent = EntityUtils.toString(entity);
        
        String regEx = "class=\"dragableBox\" id=\"box(\\d+)\">([a-zA-Z ]+)<span";
        
        Pattern pattern = Pattern.compile(regEx);
        Matcher matcher = pattern.matcher(pageContent);
        
        List<Guild> result = new LinkedList<Guild>();
        
        while (matcher.find()) {
        	Guild guild = new Guild();
        	guild.setId(matcher.group(1));
        	guild.setName(matcher.group(2).trim());
        	result.add(guild);
        }
        
        return result;
	}

	public static void predict(List<Guild> candidates, String userId,
			String userName, String password) throws ClientProtocolException, IOException {
		StringBuilder nodeIdsCSV = new StringBuilder();

		for (int i = 0; i < 8; i++) {
			nodeIdsCSV.append(candidates.get(i).getId());
			nodeIdsCSV.append(",");
		}
		
		logger.info("Ids of winners: "+candidates.toString());
		
		HttpClient httpclient = new DefaultHttpClient();
        
        HttpPost httpost = new HttpPost("https://www.guildwars.com/competitive/xunlai_tournament_house/login.php?lang=en");
        
        List <NameValuePair> nvps = new ArrayList <NameValuePair>();
		nvps.add(new BasicNameValuePair("hiddenNodeIds", nodeIdsCSV.toString()));
        nvps.add(new BasicNameValuePair("action", "insert"));
        nvps.add(new BasicNameValuePair("user_id", userId));
        nvps.add(new BasicNameValuePair("email_address", userName));
        nvps.add(new BasicNameValuePair("password", password));
        nvps.add(new BasicNameValuePair("voteBallot", "GvG"));
        nvps.add(new BasicNameValuePair("continue", "Save Predictions"));
        nvps.add(new BasicNameValuePair("submit.x", "99"));
        nvps.add(new BasicNameValuePair("submit.y", "23"));
        nvps.add(new BasicNameValuePair("submit", "Submit"));

        httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));

        HttpResponse response = httpclient.execute(httpost);
        HttpEntity entity = response.getEntity();
        entity.consumeContent();
	}

}
