package cz.zweistein.gw.predictor;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;

public class Players {
	private static Logger logger = Logger.getLogger(Players.class);
	
	private Map<String, Player> players;
	
	private Players() {
		this.players = new HashMap<String, Player>();
	}
	
	public Player getPlayerByName(String playerName) {
		Player player = this.players.get(playerName);
		
		if (player == null) {
			player = new Player();
			player.setName(playerName);
			this.players.put(playerName, player);
		}
		
		return player;
		
	}
	
	public List<Player> getListOfPredictedOrder() {
		List<Player> prediction = new ArrayList<Player>(this.players.values());
		Collections.sort(prediction);
		return prediction;
	}
	
	public static Players getPlayers () throws ClientProtocolException, IOException {
		
		Players players = new Players();
		
		Calendar calNow = new GregorianCalendar();
		Calendar calTournament = new GregorianCalendar(2008, 0, 1);
		
		while (calTournament.getTimeInMillis() < calNow.getTimeInMillis()) { 
			
			String url = "http://www.guildwars.com/competitive/tournament" 
				+ calTournament.get(Calendar.YEAR) + "/"
				+ calTournament.getDisplayName(Calendar.MONTH, Calendar.LONG, Locale.US).toLowerCase()
				+ "/1v1resultselim.php";
			
			logger.info("Loading: " + url);
		
	        HttpClient httpclient = new DefaultHttpClient();
	        HttpGet httpget = new HttpGet(url); 
	        HttpResponse response = httpclient.execute(httpget);
	        
	        HttpEntity entity = response.getEntity();
	        String pageContent = EntityUtils.toString(entity);
	        if (pageContent.indexOf("We're sorry, the page you requested could not be found.") > 0) {
	        	logger.info("Loading failed.");
	        	break;
	        }
	        String resultTable = pageContent.substring(pageContent.indexOf("Elimination Results"), pageContent.indexOf("Swiss Rounds Results"));
	        
	        String regEx= "<td>(.+)</td><td>(.+)</td>";
	        
	        Pattern pattern = Pattern.compile(regEx);
	        Matcher matcher = pattern.matcher(resultTable);
	        
	        
	        while (matcher.find()) {
	        	int order = Integer.parseInt(matcher.group(1));
	        	String playerName = matcher.group(2).trim();
	        	
	        	Player player = players.getPlayerByName(playerName);
	        	player.addOrder(order);
	        }

	        calTournament.add(Calendar.MONTH, 1);
        
		}
		
		return players;
	}
	
	public static List<Player> getPredicablePlayers(String userId, String userName, String password) throws ClientProtocolException, IOException {
		HttpClient httpclient = new DefaultHttpClient();
        
        HttpPost httpost = new HttpPost("https://www.guildwars.com/competitive/xunlai_tournament_house/login.php?lang=en");

        List <NameValuePair> nvps = new ArrayList <NameValuePair>();
        nvps.add(new BasicNameValuePair("continue", "View 1v1 Ballot"));
        nvps.add(new BasicNameValuePair("action", "insert"));
        nvps.add(new BasicNameValuePair("user_id", userId));
        nvps.add(new BasicNameValuePair("email_address", userName));
        nvps.add(new BasicNameValuePair("password", password));

        httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));

        HttpResponse response = httpclient.execute(httpost);
        HttpEntity entity = response.getEntity();
        String pageContent = EntityUtils.toString(entity);
        
        String regEx = "class=\"dragableBox\" id=\"box(\\d+)\">([a-zA-Z ]+)<span";
        
        Pattern pattern = Pattern.compile(regEx);
        Matcher matcher = pattern.matcher(pageContent);
        
        List<Player> result = new LinkedList<Player>();
        
        while (matcher.find()) {
        	Player guild = new Player();
        	guild.setId(matcher.group(1));
        	guild.setName(matcher.group(2).trim());
        	result.add(guild);
        }
        
        return result;
	}

	public static void predict(List<Player> candidates, String userId,
			String userName, String password) throws ClientProtocolException, IOException {
		StringBuilder nodeIdsCSV = new StringBuilder();

		for (int i = 0; i < 8; i++) {
			nodeIdsCSV.append(candidates.get(i).getId());
			nodeIdsCSV.append(",");
		}
		
		logger.info("Ids of chosen ones: "+candidates.toString());
		
		HttpClient httpclient = new DefaultHttpClient();
        
        HttpPost httpost = new HttpPost("https://www.guildwars.com/competitive/xunlai_tournament_house/login.php?lang=en");
        
        List <NameValuePair> nvps = new ArrayList <NameValuePair>();
		nvps.add(new BasicNameValuePair("hiddenNodeIds", nodeIdsCSV.toString()));
        nvps.add(new BasicNameValuePair("action", "insert"));
        nvps.add(new BasicNameValuePair("user_id", userId));
        nvps.add(new BasicNameValuePair("email_address", userName));
        nvps.add(new BasicNameValuePair("password", password));
        nvps.add(new BasicNameValuePair("vote_ballot", "1v1"));
        nvps.add(new BasicNameValuePair("continue", "Save Predictions"));
        nvps.add(new BasicNameValuePair("submit.x", "99"));
        nvps.add(new BasicNameValuePair("submit.y", "23"));
        nvps.add(new BasicNameValuePair("submit", "Submit"));

        httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));

        HttpResponse response = httpclient.execute(httpost);
        HttpEntity entity = response.getEntity();
        entity.consumeContent();
	}

	public boolean isKnown(String playerName) {
		return this.players.containsKey(playerName);
	}

}
