package cz.zweistein.gw.predictor;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.apache.http.client.ClientProtocolException;
import org.apache.log4j.Logger;

public class Predictor {
	private static Logger logger = Logger.getLogger(Predictor.class);
	
	
	public static void main(String[] args) throws ClientProtocolException, IOException {
		
		/*
		// 1) Load data from web
		
		Players players = Players.getPlayers();
        
		// 2) Predict stuff.
        List<Player> predictedPlayerOrder = players.getListOfPredictedOrder();
        
        // 3) Profit
        */
		
		logger.info("Xunlai Tournament House Prediction Helper.");
		logger.info("version 0.1.2 [z], 2008");

		if (args.length < 3) {
			logger.info("Usage: Predictor email@example.com password userId");
		} else {
			String userName = args[0];
			String password = args[1];
			String userId = args[2];
			
			boolean predictGvG = true;
			boolean predict1v1 = true;
			
			if (args.length >= 4) {
				if (args[3].equals("GvG")) {
					predict1v1 = false;
				} else if (args[3].equals("1v1")) {
					predictGvG = false;
				} else {
					logger.info("Unknown option "+args[3]);
				}
			}
			
			logger.info("Predicting for "+userName+" with ID "+userId);
			
			if (predict1v1) {
				logger.info("1v1 Preditions:");
				predictPlayers(userId, userName, password);
			}
			
			if (predictGvG) { 
				logger.info("GvG Preditions:");
				predictGuilds(userId, userName, password);
			}
		}
		
		logger.info("Exiting.");
	}

	private static void predictGuilds(String userId, String userName, String password) throws ClientProtocolException, IOException {
		logger.info("Checking past results for trends.");
		Guilds guilds = Guilds.getGuilds();
		logger.debug("We know about theese: " + guilds.getListOfPredictedOrder());
		logger.info("Checking to see who is qualified.");
        List<Guild> predictableGuilds = Guilds.getPredicableGuilds(userId, userName, password);
        List<Guild> candidates = new ArrayList<Guild>(predictableGuilds.size());
        logger.debug("Picking who to vote for");
        for (Iterator<Guild> iterator = predictableGuilds.iterator(); iterator.hasNext();) {
        	Guild predicableGuild = iterator.next();
			if (guilds.isKnown(predicableGuild.getName())) {
				Guild guild = guilds.getGuildByName(predicableGuild.getName());
				guild.setId(predicableGuild.getId());
				candidates.add(guild);
				iterator.remove();
			}
		}

        Collections.sort(candidates);
        
        if (candidates.size()<8) {
        	if (predictableGuilds.size()+candidates.size() >= 8) {
        		Collections.shuffle(predictableGuilds);
        		while (candidates.size()<8) {
        			candidates.add(predictableGuilds.get(0));
        			predictableGuilds.remove(0);
        		}
        	}
        }
        
        logger.info("Voting for GvG: " +candidates);
        Guilds.predict(candidates, userId, userName, password);
        logger.info("Voting for GvG done.");
	}
	
	private static void predictPlayers(String userId, String userName, String password) throws ClientProtocolException, IOException {
		logger.info("Checking past results for trends.");
		Players players = Players.getPlayers();
		logger.debug("We know about theese: " + players.getListOfPredictedOrder());
		logger.info("Checking to see who is qualified.");
        List<Player> predictablePlayers = Players.getPredicablePlayers(userId, userName, password);
        List<Player> candidates = new ArrayList<Player>(predictablePlayers.size());
        logger.info("Picking who to vote for.");
        for (Iterator<Player> iterator = predictablePlayers.iterator(); iterator.hasNext();) {
        	Player predicablePlayer = iterator.next();
			if (players.isKnown(predicablePlayer.getName())) {
				Player player = players.getPlayerByName(predicablePlayer.getName());
				player.setId(predicablePlayer.getId());
				candidates.add(player);
				iterator.remove();
			}
		}

        Collections.sort(candidates);
        
        if (candidates.size()<8) {
        	if (predictablePlayers.size()+candidates.size() >= 8) {
        		Collections.shuffle(predictablePlayers);
        		while (candidates.size()<8) {
        			candidates.add(predictablePlayers.get(0));
        			predictablePlayers.remove(0);
        		}
        	}
        }
        
        logger.info("Voting for 1v1: " +candidates);
        Players.predict(candidates, userId, userName, password);
        logger.info("Voting for 1v1 done.");
	}
	
}
